{************************************************}
{ Laser Jet Print Utility                        }
{ (c) 1993 by Valery Votintsev, Info-Link, ltd.  }
{************************************************}

program LPR;

uses
   DOS,
   Printer;

const
   FormFeed = #12;            { Page Feed }
   LF       = #13;            { Line Feed }
   MaxWord  = 65535;

var
   FileName  : string;        { Text File Name   }
   FontName  : string;        { Font File Name   }
   FontPath  : string;        { Path To the Font }
   cMode     : char;          { Font Mode: C-Compressed,}
                              {            S-Standard   }
   ModeStr   : string;        { Font Mode Substring     }
   Compressed: boolean;       { Compressed Flag    }
   F,                         { Text File Variable }
   FT        : File;          { Font File Variable }
   nPos      : byte;          { Position Number    }
   i,
   Code      : integer;
   LeftShift : Integer;       { Shift From Left Margin }
   NumRead   : Word;          { Number of bytes readed }
   buf       : array[1..MaxInt] of char;    { Buffer   }

{-----------------------------------------------}
function Space(n: byte): string;
{ Returns String consist of n spaces }
var
   i:byte;
   S:string;
begin
   S:='';
   For i:=1 to n do S:=S+' ';  { Add Space n times }
   Space:=S;
end;  { Space }

{-----------------------------------------------}
function FileExists(FileName: string): Boolean;
{ Returns True if file exists; otherwise, it returns False }
var
   DirInfo: SearchRec;
begin
   FindFirst(Filename, Archive, DirInfo);
   FileExists:=(DosError = 0);
   doserror:=0;

end;  { FileExists }

{-----------------------------------------------}
FUNCTION ExeDir : string;
{ Returns Path from program started }
var
   Tmp : string;
BEGIN
   Tmp := ParamStr(0);
   while (Tmp[Length(Tmp)] <> '\') and (Length(Tmp) <> 0) do
      Delete(Tmp,Length(Tmp),1);
   ExeDir := Tmp;
END;

{-----------------------------------------------}
{ Check Our Arguments }
Procedure ParseArgs;
begin
   Writeln('===== LPR - Print a file to Laser Jet ======');
   Writeln('(c) 1993 by Valery Votintsev, Info-Link ltd.');

   If ParamCount=0 then begin
      Writeln('Usage: LPR <filename> [/switch]');
      Writeln(' switches:  /c - compressed font');
      Writeln('            /s - standard font (default!)');
      Writeln('            /l<n> - Left Spacing <n> chars');
      Writeln('');
      Halt;
   end else begin
      cMode:='S';
      LeftShift:=0;
      Compressed:=False;
      FileName:=Paramstr(1);

      For i:=2 to ParamCount do begin
         ModeStr:=ParamStr(i);
         nPos:=Pos('/',ModeStr);
         If nPos>0 then begin
            ModeStr:=Copy(ModeStr,nPos+1,Length(ModeStr)-nPos);
            If Length(ModeStr)>0 then begin
               cMode:=UpCase(ModeStr[1]);
               ModeStr:=Copy(ModeStr,2,Length(ModeStr)-1);
               If cMode = 'C' then Compressed:=True;
               If cMode = 'L' then begin
                  Val(ModeStr,LeftShift,Code);
                  If Code<>0 then begin
                     Writeln('Invalid parameter:  /',cMode,ModeStr,'.');
                     Halt(1);
                  end;
               end;
            end;
         end;
      end;
   end;
end;

{-----------------------------------------------}
{ Read the Font from Disk and Load It To Printer }

Procedure LoadFont;
Var
   FontName:string;                 { Font File Name }
begin
   FontPath:=ExeDir;
   If Compressed then FontName:=FontPath+'LPRC.FNT'
                 else FontName:=FontPath+'LPRS.FNT';
   Assign(f,FontName);
   {$I-}
   Reset(f,1);
   {$I+}
   If IOResult<>0 then begin
      Writeln('Font  ',FontName,'  not found!');
      Halt(1);
   end else begin
      Writeln('Font loading...');
      repeat
         BlockRead(F, Buf, SizeOf(Buf), NumRead);
         For i:=1 to NumRead do Write(LST, Buf[i]);
      until (NumRead = 0);
      Close(F);
   end;
end;

{-----------------------------------------------}
Procedure LoadText;
{ Read the Text File from Disk and Load It To Printer   }
Var
   LeftStr:string;                 { Left Shift String  }
   EndOfLine:Boolean;              { EOF Flag           }
begin
   LeftStr:=Space(LeftShift);      { Fill Left Shifting }
   EndOfLine:=True;

   Assign(fT,FileName);
   {$I-}
   Reset(fT,1);
   {$I+}
   If IOResult<>0 then begin
      Writeln('ERROR:  file ',FileName,' not found.');
      Halt(1);
   end else begin
      Writeln('File printing...');

      repeat
         BlockRead(FT, Buf, SizeOf(Buf), NumRead);
         For i:=1 to NumRead do begin
            { Check for New Line & Make Left Shift }
            If EndOfLine and (LeftShift>0) then Write(LST, LeftStr);
            If Buf[i]=LF then EndOfLine:=True
                         else EndOfLine:=False;
            Write(LST, Buf[i]);  { Print current line }
         end;
      until (NumRead = 0);
      Close(FT);

      Write(LST,FormFeed); { move the printer to the beginning }
                           { of the next page                  }
   end;
end;


{-----------------------------------------------}
{ Main Program Body }
begin
  ParseArgs;                          { Argument Parsing }
  If FileExists(FileName) then begin
    LoadFont;                         { Font loading     }
    LoadText;                         { Print the file   }
  end else
    writeln('File ',FileName,' not found');
  {That's All!}
end.

